/*
	dcc_cmd.h
		
	A simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2010, 2012, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_DCC_CMD_H_
#define _INCLUDE_DCC_CMD_H_

#include <pthread.h>
#include <stdbool.h>

#define BAUDRATE B9600

#include "dcc_decoder.h"
#include "dcc_address.h"


#define DCC_FUNCTION_INTERVAL 11
#define DCC_SPEED_INTERVAL 7
#define DCC_ACCESSORY_INTERVAL 13

#define DCC_SPEED_T 4
#define DCC_FUNCTION_T 2
#define DCC_ACCESSORY_T 4


struct dcc_accessory_decoder {
	struct dcc_accessory_decoder *next;
	int refcount;

	struct dcc_address address;

	unsigned int value;
	time_t value_t;
	
	int lock;
};

extern struct dcc_accessory_decoder *dcc_accessory_decoder_list;
extern pthread_mutex_t dcc_accessory_decoder_list_mutex;

struct dcc_accessory_decoder *dcc_accessory_decoder_add(
    enum dcc_address_type type, 
    int address);
struct dcc_accessory_decoder *dcc_accessory_decoder_find(int address);
void dcc_accessory_decoder_output_set(struct dcc_accessory_decoder *decoder,
    int port, bool value);
void dcc_accessory_decoder_del(struct dcc_accessory_decoder *decoder);
void dcc_accessory_decoder_put(struct dcc_accessory_decoder *decoder);

extern bool rail_power;

int dcc_cv_set(struct dcc_address *address, int cv, unsigned char val);

#endif /* _INCLUDE_DCC_CMD_H_ */
