/*
	dcc-cmd
	
	A simple DCC command station using the dcc-kiss interface.
	
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2010, 2011, 2012, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_DCC_DECODER_H_
#define _INCLUDE_DCC_DECODER_H_

#include "dcc_address.h"

#include <time.h>
#include <pthread.h>
#include <stdbool.h>

enum dcc_speed_type {
	DCC_SPEED_TYPE_NONE,	/* No motor, only functions */
	DCC_SPEED_TYPE_14,
	DCC_SPEED_TYPE_28,
	DCC_SPEED_TYPE_128,
};


struct dcc_decoder {
	struct dcc_decoder *next;
	int refcount;

	struct dcc_address address;

	double speed;
	time_t speed_t;
	enum dcc_speed_type speed_type;
	
	int nr_functions;
	unsigned int functions;

	time_t func_t;
	unsigned int func_changed;

	int lock;
};

extern struct dcc_decoder *dcc_decoder_list;
extern pthread_mutex_t dcc_decoder_list_mutex;


#define DCC_FUNCTION_FL		0x00000010
#define DCC_FUNCTION_F1		0x00000001
#define DCC_FUNCTION_F2		0x00000002
#define DCC_FUNCTION_F3		0x00000004
#define DCC_FUNCTION_F4		0x00000008
#define DCC_FUNCTION_F5		0x00000100
#define DCC_FUNCTION_F6		0x00000200
#define DCC_FUNCTION_F7		0x00000400
#define DCC_FUNCTION_F8		0x00000800
#define DCC_FUNCTION_F9		0x00001000
#define DCC_FUNCTION_F10	0x00002000
#define DCC_FUNCTION_F11	0x00004000
#define DCC_FUNCTION_F12	0x00008000

#define DCC_FUNCTION_F13	0x00010000
#define DCC_FUNCTION_F14	0x00020000
#define DCC_FUNCTION_F15	0x00040000
#define DCC_FUNCTION_F16	0x00080000
#define DCC_FUNCTION_F17	0x00100000
#define DCC_FUNCTION_F18	0x00200000
#define DCC_FUNCTION_F19	0x00400000
#define DCC_FUNCTION_F20	0x00800000

#define DCC_FUNCTION_F21	0x01000000
#define DCC_FUNCTION_F22	0x02000000
#define DCC_FUNCTION_F23	0x04000000
#define DCC_FUNCTION_F24	0x08000000
#define DCC_FUNCTION_F25	0x10000000
#define DCC_FUNCTION_F26	0x20000000
#define DCC_FUNCTION_F27	0x40000000
#define DCC_FUNCTION_F28	0x80000000


#define DCC_FUNCTION_GROUP1(f)		((f) & 0x001f)
#define DCC_FUNCTION_GROUP2_L(f)	(((f) >> 8) & 0x000f)
#define DCC_FUNCTION_GROUP2_H(f)	(((f) >> 12) & 0x000f)
#define DCC_FUNCTION_13_20(f)		(((f) >> 16) & 0x00ff)
#define DCC_FUNCTION_21_28(f)		(((f) >> 24) & 0x00ff)


static inline bool dcc_decoder_function_get(struct dcc_decoder *decoder,
    int function_nr)
{
	unsigned int mask = 0;
	
	switch (function_nr) {
		case 28:
			mask = DCC_FUNCTION_F28;
			break;
		case 27:
			mask = DCC_FUNCTION_F27;
			break;
		case 26:
			mask = DCC_FUNCTION_F26;
			break;
		case 25:
			mask = DCC_FUNCTION_F25;
			break;
		case 24:
			mask = DCC_FUNCTION_F24;
			break;
		case 23:
			mask = DCC_FUNCTION_F23;
			break;
		case 22:
			mask = DCC_FUNCTION_F22;
			break;
		case 21:
			mask = DCC_FUNCTION_F21;
			break;
		case 20:
			mask = DCC_FUNCTION_F20;
			break;
		case 19:
			mask = DCC_FUNCTION_F19;
			break;
		case 18:
			mask = DCC_FUNCTION_F18;
			break;
		case 17:
			mask = DCC_FUNCTION_F17;
			break;
		case 16:
			mask = DCC_FUNCTION_F16;
			break;
		case 15:
			mask = DCC_FUNCTION_F15;
			break;
		case 14:
			mask = DCC_FUNCTION_F14;
			break;
		case 13:
			mask = DCC_FUNCTION_F13;
			break;
		case 12:
			mask = DCC_FUNCTION_F12;
			break;
		case 11:
			mask = DCC_FUNCTION_F11;
			break;
		case 10:
			mask = DCC_FUNCTION_F10;
			break;
		case 9:
			mask = DCC_FUNCTION_F9;
			break;
		case 8:
			mask = DCC_FUNCTION_F8;
			break;
		case 7:
			mask = DCC_FUNCTION_F7;
			break;
		case 6:
			mask = DCC_FUNCTION_F6;
			break;
		case 5:
			mask = DCC_FUNCTION_F5;
			break;
		case 4:
			mask = DCC_FUNCTION_F4;
			break;
		case 3:
			mask = DCC_FUNCTION_F3;
			break;
		case 2:
			mask = DCC_FUNCTION_F2;
			break;
		case 1:
			mask = DCC_FUNCTION_F1;
			break;
		case 0:
			mask = DCC_FUNCTION_FL;
			break;
	}
	return decoder->functions & mask;
}

static inline void dcc_decoder_function_set(struct dcc_decoder *decoder,
    int function_nr, bool value)
{
	unsigned int functions = decoder->functions;
	unsigned int mask = 0;
	
	switch (function_nr) {
		case 28:
			mask = DCC_FUNCTION_F28;
			break;
		case 27:
			mask = DCC_FUNCTION_F27;
			break;
		case 26:
			mask = DCC_FUNCTION_F26;
			break;
		case 25:
			mask = DCC_FUNCTION_F25;
			break;
		case 24:
			mask = DCC_FUNCTION_F24;
			break;
		case 23:
			mask = DCC_FUNCTION_F23;
			break;
		case 22:
			mask = DCC_FUNCTION_F22;
			break;
		case 21:
			mask = DCC_FUNCTION_F21;
			break;
		case 20:
			mask = DCC_FUNCTION_F20;
			break;
		case 19:
			mask = DCC_FUNCTION_F19;
			break;
		case 18:
			mask = DCC_FUNCTION_F18;
			break;
		case 17:
			mask = DCC_FUNCTION_F17;
			break;
		case 16:
			mask = DCC_FUNCTION_F16;
			break;
		case 15:
			mask = DCC_FUNCTION_F15;
			break;
		case 14:
			mask = DCC_FUNCTION_F14;
			break;
		case 13:
			mask = DCC_FUNCTION_F13;
			break;
		case 12:
			mask = DCC_FUNCTION_F12;
			break;
		case 11:
			mask = DCC_FUNCTION_F11;
			break;
		case 10:
			mask = DCC_FUNCTION_F10;
			break;
		case 9:
			mask = DCC_FUNCTION_F9;
			break;
		case 8:
			mask = DCC_FUNCTION_F8;
			break;
		case 7:
			mask = DCC_FUNCTION_F7;
			break;
		case 6:
			mask = DCC_FUNCTION_F6;
			break;
		case 5:
			mask = DCC_FUNCTION_F5;
			break;
		case 4:
			mask = DCC_FUNCTION_F4;
			break;
		case 3:
			mask = DCC_FUNCTION_F3;
			break;
		case 2:
			mask = DCC_FUNCTION_F2;
			break;
		case 1:
			mask = DCC_FUNCTION_F1;
			break;
		case 0:
			mask = DCC_FUNCTION_FL;
			break;
	}

	functions &= ~mask;
	if (value)
		functions |= mask;
	
	decoder->func_changed |= mask;
	decoder->func_t = time(NULL);
	
	decoder->functions = functions;
}

void dcc_decoder_speed_set(struct dcc_decoder *decoder, double speed);


struct dcc_decoder *dcc_decoder_loop(struct dcc_decoder *decoder);

void dcc_decoder_del(struct dcc_decoder *decoder);
void dcc_decoder_put(struct dcc_decoder *decoder);

struct dcc_decoder *dcc_decoder_add(
    enum dcc_address_type type, 
    int address, 
    enum dcc_speed_type speed_type,
    int nr_functions);

struct dcc_decoder *dcc_decoder_find(int address);

#endif /* _INCLUDE_DCC_DECODER_H_ */
