/*
	dcc_srcpd.c
		
	SRCP implementation for a simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
 
#include "dcc_srcpd_ga.h"

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

void dcc_srcpd_info_init_ga_str(char *buffer, struct dcc_accessory_decoder *decoder)
{
	int dec_address;
		
	dec_address = decoder->address.address;
	
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "101 INFO 1 GA %d N\n",
	    dec_address
	    );
}

void dcc_srcpd_info_term_ga_str(char *buffer, struct dcc_accessory_decoder *decoder)
{
	int dec_address;
		
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	dec_address = decoder->address.address;
	sprintf(buffer, "102 INFO 1 GA %d\n",
	    dec_address);
}


int dcc_srcpd_info_get_ga_str(char *buffer, struct dcc_accessory_decoder *decoder, int port)
{
	int dec_address;
	bool dec_value;
	int r;
		
	dec_address = decoder->address.address;
	dec_value = decoder->value & (1 << port);
	
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	r = sprintf(buffer, "100 INFO 1 GA %d %d %d\n",
	    dec_address,
	    port,
	    dec_value
	    );

	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_init_ga(FILE *fd_sock, struct dcc_accessory_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_init_ga_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_get_ga(FILE *fd_sock, struct dcc_accessory_decoder *decoder, int port)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_ga_str(buffer, decoder, port);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

void dcc_srcpd_info_init_ga_update(struct dcc_accessory_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_init_ga_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_term_ga_update(struct dcc_accessory_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_ga_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_get_ga_update(struct dcc_accessory_decoder *decoder, int port)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_ga_str(info->buffer, decoder, port);

	dcc_srcpd_info_q_add(info);
}

struct dcc_srcpd_ga_delay {
	struct dcc_accessory_decoder *decoder;
	int port;
	int delay;
};

static void *dcc_srcpd_ga_delay_helper(void *arg)
{
	struct dcc_srcpd_ga_delay *delay = arg;
	struct timespec req, rem;
	
	req.tv_sec = delay->delay / 1000;
	req.tv_nsec = (delay->delay - (req.tv_sec * 1000)) * 1000000;
	
	do { 
		nanosleep(&req, &rem);
		req = rem;
	} while (rem.tv_sec || rem.tv_nsec);
	
	dcc_accessory_decoder_output_set(delay->decoder, delay->port, 0);
	dcc_srcpd_info_get_ga_update(delay->decoder, delay->port);

	dcc_accessory_decoder_put(delay->decoder);
	free(delay);

	return NULL;
}

void dcc_srcpd_ga_delay(struct dcc_accessory_decoder *decoder, int port, int delay)
{
	struct dcc_srcpd_ga_delay *del = malloc(sizeof(struct dcc_srcpd_ga_delay));
	pthread_t thread;

	del->delay = delay;
	del->decoder = decoder;
	del->port = port;

	pthread_create(&thread, NULL, dcc_srcpd_ga_delay_helper, del);
}

