/*
	dcc_srcpd.c
		
	SRCP implementation for a simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
 
#include "dcc_srcpd_gl.h"

#include <stdio.h>
#include <math.h>


void dcc_srcpd_info_init_gl_str(char *buffer, struct dcc_decoder *decoder)
{
	int dec_version;
	int dec_speedsteps;
	int dec_nroffunctions;
	int dec_address;
		
	dec_version = decoder->address.type == DCC_ADDRESS_SHORT ? 1 : 2;
	switch (decoder->speed_type) {
		case DCC_SPEED_TYPE_14:
			dec_speedsteps = 14;
			break;
		case DCC_SPEED_TYPE_28:
			dec_speedsteps = 28;
			break;
		case DCC_SPEED_TYPE_128:
			dec_speedsteps = 126;
			break;
		case DCC_SPEED_TYPE_NONE:
		default:
			dec_speedsteps = 0;
			break;
	}
	dec_nroffunctions = 5;
	dec_address = decoder->address.address;
	
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "101 INFO 1 GL %d N %d %d %d\n",
	    dec_address,
	    dec_version,
	    dec_speedsteps,
	    dec_nroffunctions
    );
}

void dcc_srcpd_info_term_gl_str(char *buffer, struct dcc_decoder *decoder)
{
	int dec_address;
		
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	dec_address = decoder->address.address;
	sprintf(buffer, "102 INFO 1 GL %d\n",
	    dec_address);
}

int dcc_srcpd_info_get_gl_str(char *buffer, struct dcc_decoder *decoder)
{
	int dec_address;
	int dec_drivemode;
	int dec_V;
	int dec_V_max;
	int i;
	int r;
		
	dec_address = decoder->address.address;
	if (!signbit(decoder->speed))
		dec_drivemode = 1;
	else
		dec_drivemode = 0;
	
	switch (decoder->speed_type) {
		case DCC_SPEED_TYPE_14:
			dec_V = fabs(decoder->speed) * 14 + 0.5;
			dec_V_max = 14;
			break;
		case DCC_SPEED_TYPE_28:
			dec_V = fabs(decoder->speed) * 28 + 0.5;
			dec_V_max = 28;
			break;
		case DCC_SPEED_TYPE_128:
			dec_V = fabs(decoder->speed) * 126 + 0.5;
			dec_V_max = 126;
			break;
		case DCC_SPEED_TYPE_NONE:
		default:
			dec_V = 0;
			dec_V_max = 0;
			break;
	}

	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	r = sprintf(buffer, "100 INFO 1 GL %d %d %d %d",
	    dec_address,
	    dec_drivemode,
	    dec_V,
	    dec_V_max
	    );
	if (r > 0)
		buffer += r;
	for (i = 0; i < decoder->nr_functions; i++) {
		r = sprintf(buffer, " %d", dcc_decoder_function_get(decoder, i));
		if (r > 0)
			buffer += r;
	}
	r = sprintf(buffer, "\n");

	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_init_gl(FILE *fd_sock, struct dcc_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_init_gl_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_get_gl(FILE *fd_sock, struct dcc_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_gl_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

void dcc_srcpd_info_init_gl_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_init_gl_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_term_gl_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_gl_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_get_gl_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_gl_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

