/*
	dcc_srcpd.c
		
	SRCP implementation for a simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "dcc_srcpd_info_q.h"

#include <stdlib.h>
#include <pthread.h>

static struct dcc_srcpd_info *dcc_srcpd_info_q = NULL;
int dcc_srcpd_info_clients = 0;

static pthread_cond_t dcc_srcpd_info_cond = PTHREAD_COND_INITIALIZER;
static pthread_mutex_t dcc_srcpd_info_q_lock = PTHREAD_MUTEX_INITIALIZER;


struct dcc_srcpd_info *dcc_srcpd_info_q_new(void)
{
	return calloc(1, sizeof(struct dcc_srcpd_info));
}

void dcc_srcpd_info_q_add(struct dcc_srcpd_info *entry)
{
	struct dcc_srcpd_info **list;
	
	pthread_mutex_lock(&dcc_srcpd_info_q_lock);

	if (dcc_srcpd_info_clients) {
		entry->refcount = 0;
		entry->next = NULL;
	
		for (list = &dcc_srcpd_info_q; (*list); list = &(*list)->next);
		*list = entry;
	
		pthread_cond_broadcast(&dcc_srcpd_info_cond);
	}
	
	pthread_mutex_unlock(&dcc_srcpd_info_q_lock);
}

struct dcc_srcpd_info *dcc_srcpd_info_q_get(struct dcc_srcpd_info *prev)
{
	struct dcc_srcpd_info **entryp, *entry;
	
	pthread_mutex_lock(&dcc_srcpd_info_q_lock);
	
	if (prev) {
		entryp = &prev->next;
	} else {
		entryp = &dcc_srcpd_info_q;
	}
	
	while (!*entryp) {
		pthread_cond_wait(&dcc_srcpd_info_cond, &dcc_srcpd_info_q_lock);
	}
	
	entry = *entryp;
	entry->refcount++;
	
	if (prev) {
		prev->refcount--;
		if (prev->refcount == 0 && dcc_srcpd_info_q == prev) {
			dcc_srcpd_info_q = prev->next;
			
		} else {
			prev = NULL;
		}
	}

	pthread_mutex_unlock(&dcc_srcpd_info_q_lock);

	if (prev)
		free(prev);

	return entry;
}

void dcc_srcpd_info_q_put(struct dcc_srcpd_info *prev)
{
	pthread_mutex_lock(&dcc_srcpd_info_q_lock);
	
	if (prev) {
		prev->refcount--;
		if (prev->refcount == 0 && dcc_srcpd_info_q == prev) {
			dcc_srcpd_info_q = prev->next;
		} else {
			prev = NULL;
		}
	}

	pthread_mutex_unlock(&dcc_srcpd_info_q_lock);
			
	if (prev)
		free(prev);
}

void dcc_srcpd_info_q_client_add(void)
{
	pthread_mutex_lock(&dcc_srcpd_info_q_lock);
	dcc_srcpd_info_clients++;
	pthread_mutex_unlock(&dcc_srcpd_info_q_lock);
}

void dcc_srcpd_info_q_client_remove(void)
{
	pthread_mutex_lock(&dcc_srcpd_info_q_lock);
	dcc_srcpd_info_clients--;
	pthread_mutex_unlock(&dcc_srcpd_info_q_lock);
}
