/*
	dcc_srcpd.c
		
	SRCP implementation for a simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "dcc_srcpd_lock.h"
#include "dcc_decoder.h"
#include "dcc_srcpd_info_q.h"

static pthread_mutex_t dcc_srcpd_lock = PTHREAD_MUTEX_INITIALIZER;

void dcc_srcpd_info_get_lock_gl_str(char *buffer, struct dcc_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "100 INFO 1 LOCK GL %d 0 %d\n", 
	    decoder->address.address, decoder->lock);
}

void dcc_srcpd_info_term_lock_gl_str(char *buffer, struct dcc_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "102 INFO 1 LOCK GL %d\n", decoder->address.address);
}

void dcc_srcpd_info_get_lock_sm_str(char *buffer, struct dcc_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "100 INFO 1 LOCK SM %d 0 %d\n", 
	    decoder->address.address, decoder->lock);
}

void dcc_srcpd_info_term_lock_sm_str(char *buffer, struct dcc_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "102 INFO 1 LOCK SM %d\n", decoder->address.address);
}


void dcc_srcpd_info_get_lock_ga_str(char *buffer, struct dcc_accessory_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "100 INFO 1 LOCK GA %d 0 %d\n", 
	    decoder->address.address, decoder->lock);
}

void dcc_srcpd_info_term_lock_ga_str(char *buffer, struct dcc_accessory_decoder *decoder)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "102 INFO 1 LOCK GA %d\n", decoder->address.address);
}

void dcc_srcpd_info_term_lock_gl_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_lock_gl_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_get_lock_gl_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_lock_gl_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_term_lock_sm_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_lock_sm_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_get_lock_sm_update(struct dcc_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_lock_sm_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_term_lock_ga_update(struct dcc_accessory_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_lock_ga_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_get_lock_ga_update(struct dcc_accessory_decoder *decoder)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_lock_ga_str(info->buffer, decoder);

	dcc_srcpd_info_q_add(info);
}

int dcc_srcpd_lock_get_gl(FILE *fd_sock, struct dcc_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_lock_gl_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_lock_get_sm(FILE *fd_sock, struct dcc_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_lock_sm_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_lock_get_ga(FILE *fd_sock, struct dcc_accessory_decoder *decoder)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_lock_ga_str(buffer, decoder);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_lock_set_gl(FILE *fd_sock, struct dcc_decoder *decoder, int duration, int session_id)
{
	int ret = -1;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (!decoder->lock || decoder->lock == session_id) {
		decoder->lock = session_id;
		ret = 0;
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	if (!ret) {
		dcc_srcpd_info_get_lock_gl_update(decoder);
	}

	return ret;
}

int dcc_srcpd_lock_set_sm(FILE *fd_sock, struct dcc_decoder *decoder, int duration, int session_id)
{
	int ret = -1;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (!decoder->lock || decoder->lock == session_id) {
		decoder->lock = session_id;
		ret = 0;
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	if (!ret) {
		dcc_srcpd_info_get_lock_sm_update(decoder);
	}

	return ret;
}

int dcc_srcpd_lock_set_ga(FILE *fd_sock, struct dcc_accessory_decoder *decoder, int duration, int session_id)
{
	int ret = -1;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (!decoder->lock || decoder->lock == session_id) {
		decoder->lock = session_id;
		ret = 0;
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	if (!ret) {
		dcc_srcpd_info_get_lock_ga_update(decoder);
	}

	return ret;
}

bool dcc_srcpd_lock_term_gl(FILE *fd_sock, int session_id, struct dcc_decoder *decoder)
{
	bool ret = false;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (decoder->lock && decoder->lock == session_id) {
		decoder->lock = 0;
		ret = true;
		dcc_srcpd_info_term_lock_gl_update(decoder);
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	return ret;
}

bool dcc_srcpd_lock_term_sm(FILE *fd_sock, int session_id, struct dcc_decoder *decoder)
{
	bool ret = false;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (decoder->lock && decoder->lock == session_id) {
		decoder->lock = 0;
		ret = true;
		dcc_srcpd_info_term_lock_sm_update(decoder);
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	return ret;
}

bool dcc_srcpd_lock_term_ga(FILE *fd_sock, int session_id, struct dcc_accessory_decoder *decoder)
{
	bool ret = false;
	
	pthread_mutex_lock(&dcc_srcpd_lock);
	if (decoder->lock && decoder->lock == session_id) {
		decoder->lock = 0;
		ret = true;
		dcc_srcpd_info_term_lock_ga_update(decoder);
	}
	pthread_mutex_unlock(&dcc_srcpd_lock);

	return ret;
}

void dcc_srcpd_lock_session_clean(int session_id)
{
	struct dcc_decoder *entry;
	struct dcc_accessory_decoder *aentry;
	
	/* loop over decoderlist */
	pthread_mutex_lock(&dcc_accessory_decoder_list_mutex);
	for (entry = dcc_decoder_list; entry; entry = entry->next) {
		pthread_mutex_lock(&dcc_srcpd_lock);
		if (entry->lock && entry->lock == session_id) {
			entry->lock = 0;
			
			dcc_srcpd_info_term_lock_gl_update(entry);
		}
		pthread_mutex_unlock(&dcc_srcpd_lock);
	}
	pthread_mutex_unlock(&dcc_accessory_decoder_list_mutex);

	/* loop over decoderlist */
	pthread_mutex_lock(&dcc_accessory_decoder_list_mutex);
	for (aentry = dcc_accessory_decoder_list; aentry; aentry = aentry->next) {
		pthread_mutex_lock(&dcc_srcpd_lock);
		if (aentry->lock && aentry->lock == session_id) {
			aentry->lock = 0;
			
			dcc_srcpd_info_term_lock_ga_update(aentry);
		}
		pthread_mutex_unlock(&dcc_srcpd_lock);
	}
	pthread_mutex_unlock(&dcc_accessory_decoder_list_mutex);
}
