/*
	dcc_srcpd.c
		
	SRCP implementation for a simple DCC command station

	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
 
#include "dcc_srcpd_power.h"
#include "dcc_cmd.h"
#include "dcc_srcpd_info_q.h"

#include <stdio.h>

void dcc_srcpd_info_init_power_str(char *buffer)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "101 INFO 1 POWER\n");
}

void dcc_srcpd_info_term_power_str(char *buffer)
{
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	sprintf(buffer, "102 INFO 1 POWER\n");
}

int dcc_srcpd_info_get_power_str(char *buffer)
{
	int r;
		
	buffer = dcc_srcpd_rep_timestamp_str(buffer);
	r = sprintf(buffer, "100 INFO 1 POWER %s",
	    rail_power ? DCC_SRCPD_ON : DCC_SRCPD_OFF);
	if (r > 0)
		buffer += r;
	r = sprintf(buffer, "\n");

	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_get_power(FILE *fd_sock)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_get_power_str(buffer);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

int dcc_srcpd_info_init_power(FILE *fd_sock)
{
	int r;
	char buffer[DCC_SRCPD_REQ_LEN];
	
	dcc_srcpd_info_init_power_str(buffer);
	
	r = fprintf(fd_sock, buffer);
	
	if (r < 0)
		return -1;
	else
		return 0;
}

void dcc_srcpd_info_get_power_update(void)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_get_power_str(info->buffer);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_init_power_update(void)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_init_power_str(info->buffer);

	dcc_srcpd_info_q_add(info);
}

void dcc_srcpd_info_term_power_update(void)
{
	struct dcc_srcpd_info *info;

	info = dcc_srcpd_info_q_new();
	
	dcc_srcpd_info_term_power_str(info->buffer);

	dcc_srcpd_info_q_add(info);
}
